@@ -1,6 +1,6 @@
 /* -*-c++-*- */
 /* osgEarth - Dynamic map generation toolkit for OpenSceneGraph
-* Copyright 2015 Pelican Mapping
+* Copyright 2016 Pelican Mapping
 * http://osgearth.org
 *
 * osgEarth is free software; you can redistribute it and/or modify
@@ -37,6 +37,8 @@ using namespace geos;
 using namespace geos::operation;
 #endif
 
+#define GEOS_OUT OE_DEBUG
+
 #define LC "[Geometry] "
 
 
@@ -105,7 +107,7 @@ Geometry::cloneAs( const Geometry::Type& newType ) const
 }
 
 osg::Vec3Array*
-Geometry::toVec3Array() const 
+Geometry::createVec3Array() const 
 {
     osg::Vec3Array* result = new osg::Vec3Array( this->size() );
     std::copy( begin(), end(), result->begin() );
@@ -113,7 +115,7 @@ Geometry::toVec3Array() const
 }
 
 osg::Vec3dArray*
-Geometry::toVec3dArray() const 
+Geometry::createVec3dArray() const 
 {
     osg::Vec3dArray* result = new osg::Vec3dArray( this->size() );
     std::copy( begin(), end(), result->begin() );
@@ -251,8 +253,14 @@ Geometry::crop( const Polygon* cropPoly, osg::ref_ptr<Geometry>& output ) const
                 cropGeom,
                 overlay::OverlayOp::opINTERSECTION );
         }
+        catch (const geos::util::TopologyException& ex) {
+            GEOS_OUT << LC << "Crop(GEOS): "
+                << (ex.what()? ex.what() : " no error message")
+                << std::endl;
+            outGeom = 0L;
+        }
         catch(const geos::util::GEOSException& ex) {
-            OE_NOTICE << LC << "Crop(GEOS): "
+            OE_INFO << LC << "Crop(GEOS): "
                 << (ex.what()? ex.what() : " no error message")
                 << std::endl;
             outGeom = 0L;
@@ -302,6 +310,18 @@ Geometry::crop( const Polygon* cropPoly, osg::ref_ptr<Geometry>& output ) const
 #endif // OSGEARTH_HAVE_GEOS
 }
 
+bool
+Geometry::crop( const Bounds& bounds, osg::ref_ptr<Geometry>& output ) const
+{
+    osg::ref_ptr<Polygon> poly = new Polygon;
+    poly->resize( 4 );        
+    (*poly)[0].set(bounds.xMin(), bounds.yMin(), 0);
+    (*poly)[1].set(bounds.xMax(), bounds.yMin(), 0);
+    (*poly)[2].set(bounds.xMax(), bounds.yMax(), 0);
+    (*poly)[3].set(bounds.xMin(), bounds.yMax(), 0);
+    return crop(poly, output);
+}
+
 bool
 Geometry::geounion( const Geometry* other, osg::ref_ptr<Geometry>& output ) const
 {
@@ -324,8 +344,14 @@ Geometry::geounion( const Geometry* other, osg::ref_ptr<Geometry>& output ) cons
                 otherGeom,
                 overlay::OverlayOp::opUNION );
         }
+        catch (const geos::util::TopologyException& ex) {
+            GEOS_OUT << LC << "Crop(GEOS): "
+                << (ex.what()? ex.what() : " no error message")
+                << std::endl;
+            outGeom = 0L;
+        }
         catch(const geos::util::GEOSException& ex) {
-            OE_NOTICE << LC << "Union(GEOS): "
+            OE_INFO << LC << "Union(GEOS): "
                 << (ex.what()? ex.what() : " no error message")
                 << std::endl;
             outGeom = 0L;
@@ -395,8 +421,14 @@ Geometry::difference( const Polygon* diffPolygon, osg::ref_ptr<Geometry>& output
                 diffGeom,
                 overlay::OverlayOp::opDIFFERENCE );
         }
+        catch (const geos::util::TopologyException& ex) {
+            GEOS_OUT << LC << "Crop(GEOS): "
+                << (ex.what()? ex.what() : " no error message")
+                << std::endl;
+            outGeom = 0L;
+        }
         catch(const geos::util::GEOSException& ex) {
-            OE_NOTICE << LC << "Diff(GEOS): "
+            OE_INFO << LC << "Diff(GEOS): "
                 << (ex.what()? ex.what() : " no error message")
                 << std::endl;
             outGeom = 0L;
@@ -428,6 +460,35 @@ Geometry::difference( const Polygon* diffPolygon, osg::ref_ptr<Geometry>& output
 #endif // OSGEARTH_HAVE_GEOS
 }
 
+bool
+Geometry::intersects(
+            const class Geometry* other
+            ) const
+{
+#ifdef OSGEARTH_HAVE_GEOS
+
+    GEOSContext gc;
+
+    //Create the GEOS Geometries
+    geom::Geometry* inGeom   = gc.importGeometry( this );
+    geom::Geometry* otherGeom = gc.importGeometry( other );
+
+    bool intersects = inGeom->intersects( otherGeom );
+
+    //Destroy the geometry
+    gc.disposeGeometry( otherGeom );
+    gc.disposeGeometry( inGeom );
+
+    return intersects;
+
+#else // OSGEARTH_HAVE_GEOS
+
+    OE_WARN << LC << "Intersects failed - GEOS not available" << std::endl;
+    return false;
+
+#endif // OSGEARTH_HAVE_GEOS
+}
+
 osg::Vec3d
 Geometry::localize()
 {
@@ -497,6 +558,30 @@ void Geometry::removeDuplicates()
     }
 }
 
+void
+Geometry::removeColinearPoints()
+{
+    if ( size() >= 3 )
+    {
+        std::vector<unsigned> ind;
+
+        for(unsigned i=0; i<size()-2; ++i)
+        {
+            osg::Vec3d v0( at(i+1) - at(i) );
+            v0.normalize();
+            osg::Vec3d v1( at(i+2) - at(i) );
+            v1.normalize();
+            if ( osg::equivalent(v0*v1, 1.0) )
+                ind.push_back(i+1);
+        }
+
+        for(std::vector<unsigned>::reverse_iterator r = ind.rbegin(); r != ind.rend(); ++r)
+        {
+            erase( begin() + (*r) );
+        }
+    }
+}
+
 Geometry::Orientation 
 Geometry::getOrientation() const
 {
@@ -785,6 +870,14 @@ Polygon::removeDuplicates()
         (*i)->removeDuplicates();
 }
 
+void
+Polygon::removeColinearPoints()
+{
+    Ring::removeColinearPoints();
+    for( RingCollection::const_iterator i = _holes.begin(); i != _holes.end(); ++i )
+        (*i)->removeColinearPoints();
+}
+
 //----------------------------------------------------------------------------
 
 MultiGeometry::MultiGeometry( const MultiGeometry& rhs ) :
@@ -886,6 +979,15 @@ MultiGeometry::rewind( Orientation orientation )
     }
 }
 
+void
+MultiGeometry::removeColinearPoints()
+{
+    for( GeometryCollection::const_iterator i = _parts.begin(); i != _parts.end(); ++i )
+    {
+        i->get()->removeColinearPoints();
+    }
+}
+
 //----------------------------------------------------------------------------
 
 GeometryIterator::GeometryIterator( Geometry* geom, bool holes ) :
